@@ -2,10 +2,7 @@ import numpy as np
 import pytest
 
 from merlin_standard_lib import ColumnSchema, Schema, Tag
-from transformers4rec.data.preprocessing import (
-    add_item_first_seen_col_to_df,
-    remove_consecutive_interactions,
-)
+from transformers4rec.data import preprocessing
 from transformers4rec.data.synthetic import (
     generate_item_interactions,
     synthetic_ecommerce_data_schema,
@@ -21,7 +18,7 @@ def test_remove_consecutive_interactions():
     schema += Schema([ColumnSchema.create_continuous("timestamp", tags=[Tag.SESSION])])
 
     interactions_df = generate_item_interactions(500, schema)
-    filtered_df = remove_consecutive_interactions(interactions_df.copy())
+    filtered_df = preprocessing.remove_consecutive_interactions(interactions_df.copy())
 
     assert len(filtered_df) < len(interactions_df)
     assert len(filtered_df) == 499
@@ -32,7 +29,7 @@ def test_add_item_first_seen_col_to_df():
     schema = synthetic_ecommerce_data_schema.remove_by_name("item_recency")
     schema += Schema([ColumnSchema.create_continuous("timestamp", tags=[Tag.SESSION])])
 
-    df = add_item_first_seen_col_to_df(generate_item_interactions(500, schema))
+    df = preprocessing.add_item_first_seen_col_to_df(generate_item_interactions(500, schema))
 
     assert len(list(df.columns)) == len(schema) + 1
     assert isinstance(df["item_ts_first"], pd.Series)
