@@ -23,6 +23,7 @@ import torchmetrics as tm
 
 from ..block.base import Block, BuildableBlock, SequentialBlock
 from ..block.mlp import MLPBlock
+from ..masking import MaskedLanguageModeling
 from ..ranking_metric import AvgPrecisionAt, NDCGAt, RecallAt
 from ..utils.torch_utils import LambdaModule
 from .base import BlockType, PredictionTask
@@ -45,9 +46,9 @@ class BinaryClassificationPrepareBlock(BuildableBlock):
 class BinaryClassificationTask(PredictionTask):
     DEFAULT_LOSS = torch.nn.BCELoss()
     DEFAULT_METRICS = (
-        tm.Precision(num_classes=2),
-        tm.Recall(num_classes=2),
-        tm.Accuracy(),
+        tm.Precision(num_classes=2, task="binary"),
+        tm.Recall(num_classes=2, task="binary"),
+        tm.Accuracy(task="binary"),
         # TODO: Fix this: tm.AUC()
     )
 
@@ -135,9 +136,6 @@ class NextItemPredictionTask(PredictionTask):
         pad token id.
     target_dim: int
         vocabulary size of item ids
-    hf_format: bool
-        Output the dictionary of outputs needed by RecSysTrainer, if set to False,
-        return the predictions tensor.
     """
 
     DEFAULT_METRICS = (
@@ -157,14 +155,12 @@ class NextItemPredictionTask(PredictionTask):
         softmax_temperature: float = 1,
         padding_idx: int = 0,
         target_dim: int = None,
-        hf_format=False,
     ):
         super().__init__(loss=loss, metrics=metrics, task_block=task_block, task_name=task_name)
         self.softmax_temperature = softmax_temperature
         self.weight_tying = weight_tying
         self.padding_idx = padding_idx
         self.target_dim = target_dim
-        self.hf_format = hf_format
 
         self.item_embedding_table = None
         self.masking = None
@@ -216,12 +212,7 @@ class NextItemPredictionTask(PredictionTask):
             body, input_size, device=device, inputs=inputs, task_block=task_block, pre=pre
         )
 
-    def forward(
-        self, inputs: torch.Tensor, ignore_masking=True, training=False, hf_format=None, **kwargs
-    ):
-        if hf_format is not None:
-            self.hf_format = hf_format
-
+    def forward(self, inputs: torch.Tensor, targets=None, training=False, testing=False, **kwargs):
         if isinstance(inputs, (tuple, list)):
             inputs = inputs[0]
         x = inputs.float()
@@ -230,28 +221,14 @@ class NextItemPredictionTask(PredictionTask):
             x = self.task_block(x)  # type: ignore
 
         # Retrieve labels from masking
-        if training or not ignore_masking:
+        if training or testing:
             labels = self.masking.masked_targets  # type: ignore
             trg_flat = labels.flatten()
             non_pad_mask = trg_flat != self.padding_idx
             labels_all = torch.masked_select(trg_flat, non_pad_mask)
             # remove padded items, keep only masked positions
             x = self.remove_pad_3d(x, non_pad_mask)
-        else:
-            # keep only last non-padded position for the 'Prediction step'
-            labels = self.embeddings.item_seq
-            non_pad_mask = labels != self.padding_idx
-            last_item_sessions = non_pad_mask.sum(dim=1) - 1
-            rows_ids = torch.arange(labels.size(0), dtype=torch.long, device=labels.device)
-            labels_all = labels[rows_ids, last_item_sessions]
-            labels_all = labels_all.flatten()
-            x = x[rows_ids, last_item_sessions]
-
-        # Compute predictions probs
-        x = self.pre(x)  # type: ignore
-
-        # prepare outputs for HF trainer
-        if self.hf_format:
+            x = self.pre(x)  # type: ignore
             loss = self.loss(x, labels_all)
             return {
                 "loss": loss,
@@ -260,7 +237,19 @@ class NextItemPredictionTask(PredictionTask):
                 # "pred_metadata": {},
                 # "model_outputs": [],
             }
-            # TODO: Add model_outputs and metadata
+        else:
+            # Get the hidden position to use for predicting the next item
+            labels = self.embeddings.item_seq
+            non_pad_mask = labels != self.padding_idx
+            rows_ids = torch.arange(labels.size(0), dtype=torch.long, device=labels.device)
+            if isinstance(self.masking, MaskedLanguageModeling):
+                last_item_sessions = non_pad_mask.sum(dim=1)
+            else:
+                last_item_sessions = non_pad_mask.sum(dim=1) - 1
+            x = x[rows_ids, last_item_sessions]
+
+        # Compute predictions probs
+        x = self.pre(x)  # type: ignore
 
         return x
 
@@ -273,18 +262,11 @@ class NextItemPredictionTask(PredictionTask):
         out_tensor = inp_tensor_fl.view(-1, inp_tensor.size(1))
         return out_tensor
 
-    def calculate_metrics(  # type: ignore
-        self, predictions, targets, mode="val", forward=True, **kwargs
-    ) -> Dict[str, torch.Tensor]:
+    def calculate_metrics(self, predictions, targets) -> Dict[str, torch.Tensor]:  # type: ignore
         if isinstance(targets, dict) and self.target_name:
             targets = targets[self.target_name]
 
         outputs = {}
-        if forward:
-            predictions = self(predictions, ignore_masking=False)
-            if self.hf_format:
-                targets = predictions["labels"]
-                predictions = predictions["predictions"]
         predictions = self.forward_to_prediction_fn(predictions)
 
         for metric in self.metrics:
