@@ -19,6 +19,7 @@ from abc import abstractmethod
 
 import torch
 import torchmetrics as tm
+from torchmetrics.utilities.data import dim_zero_cat
 
 from merlin_standard_lib import Registry
 
@@ -55,7 +56,7 @@ class RankingMetric(tm.Metric):
 
     def compute(self):
         # Computing the mean of the batch metrics (for each cut-off at topk)
-        return torch.cat(self.metric_mean, axis=0).mean(0)
+        return dim_zero_cat(self.metric_mean).mean(0)
 
     @abstractmethod
     def _metric(self, ks: torch.Tensor, preds: torch.Tensor, target: torch.Tensor) -> torch.Tensor:
@@ -279,9 +280,9 @@ class NDCGAt(RankingMetric):
 
 
 @ranking_metrics_registry.register_with_multiple_names("mrr_at", "mrr")
-class MeanRecipricolRankAt(RankingMetric):
+class MeanReciprocalRankAt(RankingMetric):
     def __init__(self, top_ks=None, labels_onehot=False):
-        super(MeanRecipricolRankAt, self).__init__(top_ks=top_ks, labels_onehot=labels_onehot)
+        super(MeanReciprocalRankAt, self).__init__(top_ks=top_ks, labels_onehot=labels_onehot)
 
     def _metric(
         self, ks: torch.Tensor, scores: torch.Tensor, labels: torch.Tensor, log_base: int = 2
@@ -309,6 +310,8 @@ class MeanRecipricolRankAt(RankingMetric):
             device=scores.device, dtype=torch.float32
         )
         for index, k in enumerate(ks):
-            values, _ = (topk_labels[:, :k] / (torch.arange(k) + 1)).max(dim=1)
+            values, _ = (topk_labels[:, :k] / (torch.arange(k) + 1).to(device=scores.device)).max(
+                dim=1
+            )
             results[:, index] = values
         return results
